@@ -13,8 +13,10 @@ use File::HomeDir ();
 use File::Slurp ();
 use File::Spec ();
 use IO::File ();
+use IO::Prompt ();
+use Text::Balanced ();
 
-our $VERSION = '0.47';
+our $VERSION = '0.48';
 
 use constant LEADCHAR => '* ';
 
@@ -32,6 +34,7 @@ sub new {
                                   Exec_Makefile       => $params{Exec_Makefile}       || 0,
                                   Verbose             => $params{Verbose}             || 0,
                                   Debug               => $params{Debug}               || 0,
+                                  Process_Code        => $params{Process_Code}        || 0,
                                   Use_Native_Order    => $params{Use_Native_Order}    || 0,
                                   Len_Indent          => $params{Len_Indent}          || 3,
                                   DD_Indent           => $params{DD_Indent}           || 2,
@@ -46,10 +49,10 @@ sub new {
 sub convert {
     my $self = shift;
 
-    unless ($self->{Config}{reinit} || defined @{$self->{dirs}}) {
+    unless ($self->{Config}{reinit} || @{$self->{dirs}||[]}) {
         if ($self->{Config}{Path}) {
-            my ($basename, $dirname) = File::Basename::fileparse($self->{Config}{Path});
             if (-f $self->{Config}{Path}) {
+                my ($basename, $dirname)     = File::Basename::fileparse($self->{Config}{Path});
                 $self->{Config}{Makefile_PL} = $basename;
                 $self->{Config}{Path}        = $dirname;
             }
@@ -67,27 +70,31 @@ sub convert {
             $self->{have_single_dir} = 1;
         }
     }
+
     my $Makefile_PL = File::Basename::basename($self->{Config}{Makefile_PL});
     my $Build_PL    = File::Basename::basename($self->{Config}{Build_PL});
     my $MANIFEST    = File::Basename::basename($self->{Config}{MANIFEST});
 
-    push @{$self->{dirs}}, $self->{current_dir} 
-      if @{$self->{dirs}} == 0 and $self->{Config}{reinit};
+    unshift @{$self->{dirs}}, $self->{current_dir} if $self->{Config}{reinit};
 
     $self->{show_summary} = 1 if @{$self->{dirs}} > 1;
 
-    while (my $dir = shift @{$self->{dirs}} ) {
+    while (my $dir = shift @{$self->{dirs}}) {
         $self->{current_dir} = $dir;
 
-        unless ($self->{have_single_dir}) { 
-            $" = "\n";
-            $self->_do_verbose(<<TITLE) if @{$self->{dirs}};
+        %{$self->{make_args}} = ();
+
+        unless ($self->{have_single_dir}) {
+            local $" = "\n";
+            $self->_do_verbose(<<TITLE) if !$self->{Config}{reinit};
 Remaining dists:
 ----------------
+$dir
 @{$self->{dirs}}
 
 TITLE
         }
+
         $dir = File::Spec->catfile($self->{Config}{Path}, $dir) if !$self->{have_single_dir};
         $self->{Config}{Makefile_PL} = File::Spec->catfile($dir, $Makefile_PL);
         $self->{Config}{Build_PL}    = File::Spec->catfile($dir, $Build_PL);
@@ -95,18 +102,30 @@ TITLE
 
         unless ($self->{Config}{reinit}) {
             no warnings 'uninitialized';
+
             $self->_do_verbose(LEADCHAR."Converting $self->{Config}{Makefile_PL} -> $self->{Config}{Build_PL}\n");
+
+            my $skip_msg  = LEADCHAR."Skipping $self->{Config}{Path}\n";
+               $skip_msg .= "\n" if @{$self->{dirs}};
+
             $self->_create_rcfile if $self->{Config}{Create_RC};
-            next if !$self->_exists_overwrite;
-            next if !$self->_makefile_ok;
+
+            if (!$self->_exists_overwrite || !$self->_makefile_ok) {
+                $self->_do_verbose($skip_msg);
+                next;
+            }
+
             $self->_get_data;
         }
+
         $self->_extract_args;
+        $self->_register_summary;
         $self->_convert;
         $self->_dump;
         $self->_write;
         $self->_add_to_manifest if -e $self->{Config}{MANIFEST};
     }
+
     $self->_show_summary if $self->{show_summary};
 }
 
@@ -114,19 +133,24 @@ sub _exists_overwrite {
     my $self = shift;
 
     if (-e $self->{Config}{Build_PL}) {
+        print "$self->{current_dir}:\n"
+          if $self->{show_summary} && !$self->{Config}{Verbose};
+
         print "\n" if $self->{Config}{Verbose};
         print 'A Build.PL exists already';
 
         if ($self->{Config}{Dont_Overwrite_Auto}) {
             print ".\n";
-            print 'Shall I overwrite it? [y/n] ';
-            chomp(my $input = <STDIN>);
+            my $input_ok = IO::Prompt::prompt -yn, 'Shall I overwrite it? ';
 
-            unless ($input =~ /y/i) {
+            if (!$input_ok) {
+                print "Skipped...\n";
+                print "\n" if $self->{Config}{Verbose};
                 push @{$self->{summary}{skipped}}, $self->{current_dir};
                 return 0;
+            } else {
+                print "\n" if $self->{Config}{Verbose};
             }
-            print "\n" if $self->{Config}{Verbose};
         } else {
             print ", continuing...\n";
         }
@@ -144,8 +168,8 @@ sub _create_rcfile {
         die "$rcfile exists\n";
     } else {
         my $data = $self->_parse_data('create_rc');
-        my $fh = IO::File->new(">$rcfile") or die "Can't open $rcfile: $!\n";
-        print $fh $data;
+        my $fh = IO::File->new($rcfile, '>') or die "Can't open $rcfile: $!\n";
+        print {$fh} $data;
         $fh->close;
         print LEADCHAR."Created $rcfile\n";
         exit;
@@ -180,7 +204,7 @@ sub _makefile_ok {
     }
 
     if (!$failed && $makefile =~ /WriteMakefile\(\s*%\w+.*\s*\)/s && !$self->{Config}{Exec_Makefile}) {
-        print LEADCHAR."Indirect arguments to WriteMakefile() via hash detected, setting executing mode\n";
+        $self->_do_verbose(LEADCHAR."Indirect arguments to WriteMakefile() via hash detected, setting executing mode\n");
         $self->{Config}{Exec_Makefile} = 1;
     }
 
@@ -239,7 +263,7 @@ sub _parse_data {
     } else {
         local $/ = '__END__';
         $data = <DATA>;
-        chomp($data);
+        chomp $data;
     }
 
     unless ($create_rc) {
@@ -251,8 +275,8 @@ sub _parse_data {
     unless ($create_rc) {
         # superfluosity
         shift @data_parsed;
-        chomp($data_parsed[-1]);
- 
+        chomp $data_parsed[-1];
+
         foreach my $line (split /\n/, $data_parsed[0]) {
             next unless $line;
 
@@ -271,19 +295,24 @@ sub _parse_data {
 sub _extract_args {
     my $self = shift;
 
-    push @{$self->{summary}{succeeded}}, $self->{current_dir};
-
-    push @{$self->{summary}{$self->{Config}{Exec_Makefile} ? 'method_execute' : 'method_parse'}},
-           $self->{current_dir};
-
     if ($self->{Config}{Exec_Makefile}) {
-        $self->_do_verbose(LEADCHAR."Executing $self->{Config}{Makefile_PL}\n"); 
+        $self->_do_verbose(LEADCHAR."Executing $self->{Config}{Makefile_PL}\n");
         $self->_run_makefile;
     } else {
         $self->_parse_makefile;
     }
+}
 
-    $self->{Config}{Exec_Makefile} = $self->{Config}{reinit} = 0;
+sub _register_summary {
+    my $self = shift;
+
+    push @{$self->{summary}->{succeeded}}, $self->{current_dir};
+
+    push @{$self->{summary}{$self->{Config}{Exec_Makefile} ? 'method_execute' : 'method_parse'}},
+           $self->{current_dir};
+
+    $self->{Config}{Exec_Makefile} =
+           $self->{Config}{reinit} = 0;
 }
 
 sub _run_makefile {
@@ -291,7 +320,7 @@ sub _run_makefile {
     no warnings 'redefine';
 
     *ExtUtils::MakeMaker::WriteMakefile = sub {
-      %{$self->{make_args}} = @{$self->{make_args_arr}} = @_;
+      %{$self->{make_args}{args}} = @{$self->{make_args_arr}} = @_;
     };
 
     # beware, do '' overwrites existing globals
@@ -308,7 +337,7 @@ sub _save_globals {
     $makefile =~ s/.*WriteMakefile\(\s*?(.*?)\);.*/$1/s;
 
     while ($makefile =~ s/\$(\w+)//) {
-        push @vars, $1 if defined(${$1});
+        push @vars, $1 if defined ${$1};
     }
 
     no strict 'refs';
@@ -329,164 +358,340 @@ sub _restore_globals {
 
 sub _parse_makefile {
     my $self = shift;
-    my (@histargs, %makeargs, $makefile, %trapped_loop);
+
+    $self->_parse_init;
+
+    ($self->{parse}{makefile}, $self->{make_code}{begin}, $self->{make_code}{end}) = $self->_read_makefile;
+    my ($found_string, $found_array, $found_hash) = $self->_parse_regexps;
+
+    $self->_debug(LEADCHAR."Entering parse\n\n", 'no_wait');
+
+    while ($self->{parse}{makefile}) {
+        $self->{parse}{makefile} .= "\n" 
+          unless $self->{parse}{makefile} =~ /\n$/s;
+
+        # process string
+        if ($self->{parse}{makefile} =~ s/$found_string//) {
+            $self->_parse_process_string($1,$2,$3);
+            $self->_parse_register_comment;
+            $self->_debug($self->_debug_string_text);
+        # process array
+        } elsif ($self->{parse}{makefile} =~ s/$found_array//s) {
+            $self->_parse_process_array($1,$2,$3);
+            $self->_parse_register_comment;
+            $self->_debug($self->_debug_array_text);
+        # process hash
+        } elsif ($self->{parse}{makefile} =~ s/$found_hash//s) {
+            $self->_parse_process_hash($1,$2,$3);
+            $self->_parse_register_comment;
+            $self->_debug($self->_debug_hash_text);
+        # process "code"
+        } else {
+            chomp $self->{parse}{makefile};
+
+            $self->_parse_process_code;
+            $self->_parse_catch_trapped_loop;
+
+            if ($self->{Config}{Process_Code}) {
+                $self->_parse_substitute_makeargs;
+                $self->_parse_append_makecode;
+                $self->_debug($self->_debug_code_text);
+            }
+        }
+
+        $self->{parse}{makefile} = ''
+          unless $self->{parse}{makefile} =~ /\w/;
+    }
+
+    $self->_debug(LEADCHAR."Leaving parse\n\n", 'no_wait');
+
+    %{$self->{make_args}{args}} = %{$self->{parse}{makeargs}};
+}
+
+sub _parse_init {
+    my $self = shift;
+
+    %{$self->{make_code}} = ();
+    %{$self->{parse}}     = ();
+}
+
+sub _parse_regexps {
+    my $self = shift;
 
     my $found_string = qr/^
                             \s* 
                             ['"]? (\w+) ['"]?
-                            \s* => \s* [^ \{ \[ ]
+                            \s* => \s* (?![ \{ \[ ])
                             ['"]? ([\$ \@ \% \< \> \( \) \\ \/ \- \: \. \w]+.*?) ['"]?
-                            (?: ,? \n? | ,? (\s+ \# \s+ \w+ .*?) \n)
+                            ,? ([^\n]+ \# \s+ \w+ .*?)? \n
                        /sx;
     my $found_array  = qr/^
                             \s*
                             ['"]? (\w+) ['"]?
                             \s* => \s*
                             \[ \s* (.*?) \s* \]
-                            (?: ,? \n | ,? (\s+ \# \s+ \w+ .*?) \n)
+                            ,? ([^\n]+ \# \s+ \w+ .*?)? \n
                        /sx;
     my $found_hash   = qr/^
                             \s*
                             ['"]? (\w+) ['"]?
                             \s* => \s*
                             \{ \s* (.*?) \s*? \}
-                            (?: ,? \n? | ,? (\s+ \# \s+ \w+ .*?) \n)
+                            ,? ([^\n]+ \# \s+ \w+ .*?)? \n
                        /sx;
 
-    ($makefile, $self->{make_code}{begin}, $self->{make_code}{end}) = $self->_read_makefile;
-    $self->_debug(LEADCHAR."Entering parse\n\n", 'no_wait');
+    return ($found_string, $found_array, $found_hash);
+}
 
-    while ($makefile) {
-        if ($makefile =~ s/$found_string//) {
-            my ($arg, $value, $comment) = ($1,$2,$3);
-            $comment ||= '';
-            $value =~ tr/['"]//d;
-            $makeargs{$arg} = $value;
-            push @histargs, $arg;
+sub _parse_process_string {
+    my ($self, $arg, $value, $comment) = @_;
 
-            if (defined($comment) && defined($self->{Data}{table}{$arg})) {
-                $self->{make_comments}{$self->{Data}{table}{$arg}} = $comment;
-            }
+    $value   ||= '';
+    $comment ||= '';
 
-            $self->_debug(<<DEBUG);
+    $value =~ s/^['"]//;
+    $value =~ s/['"]$//;
+
+    $self->{parse}{makeargs}{$arg} = $value;
+    push @{$self->{parse}{histargs}}, $arg;
+
+    $self->{parse}{arg}     = $arg;
+    $self->{parse}{value}   = $value;
+    $self->{parse}{comment} = $comment;
+}
+
+sub _parse_process_array {
+    my ($self, $arg, $values, $comment) = @_;
+
+    $comment ||= '';
+
+    $self->{parse}{makeargs}{$arg} = [ map { tr/['",]//d; $_ } split /,\s*/, $values ];
+    push @{$self->{parse}{histargs}}, $arg;
+
+    $self->{parse}{arg}     = $arg;
+    $self->{parse}{values}  = $self->{parse}{makeargs}{$arg},
+    $self->{parse}{comment} = $comment;
+}
+
+
+sub _parse_process_hash {
+    my ($self, $arg, $values, $comment) = @_;
+
+    $comment ||= '';
+
+    my @values_debug = split /,\s*/, $values;
+    my @values;
+
+    foreach my $value (@values_debug) {
+        push @values, map { tr/['",]//d; $_ } split /\s*=>\s*/, $value;
+    }
+
+    @values_debug = map { "$_\n        " } @values_debug;
+
+    $self->{parse}{makeargs}{$arg} = { @values };
+    push @{$self->{parse}{histargs}}, $arg;
+
+    $self->{parse}{arg}     = $arg;
+    $self->{parse}{values}  = \@values_debug,
+    $self->{parse}{comment} = $comment;
+}
+
+sub _parse_process_code {
+    my $self = shift;
+
+    my ($debug_desc, $retval);
+
+    my @code     = Text::Balanced::extract_codeblock($self->{parse}{makefile}, '()');
+    my @variable = Text::Balanced::extract_variable($self->{parse}{makefile});
+
+    # [0] extracted
+    # [1] remainder
+
+    if ($code[0]) {
+        $code[0] =~ s/^\s*\(\s*//s;
+        $code[0] =~ s/\s*\)\s*$//s;
+
+        $code[0] =~ s/\s*=>\s*/\ =>\ /gs;
+        $code[1] =~ s/^\s*,//;
+
+        $self->{parse}{makefile} = $code[1];
+        $retval                  = $code[0];
+
+        $debug_desc = 'code';
+    } elsif ($variable[0]) {
+        $self->{parse}{makefile} = $variable[1];
+        $retval                  = $variable[0];
+
+        $debug_desc = 'variable';
+    } elsif ($self->{parse}{makefile} =~ /\#/) {
+        my $comment;
+
+        $self->{parse}{makefile} .= "\n"
+          unless $self->{parse}{makefile} =~ /\n$/s;
+
+        while ($self->{parse}{makefile} =~ /\G(\s*?\#.*?\n)/cgs) {
+            $comment .= $1;
+        }
+
+        $comment ||= '';
+
+        my $quoted_comment = quotemeta $comment;
+        $self->{parse}{makefile} =~ s/$quoted_comment//s; 
+
+        my @comment;
+
+        @comment = split /\n/,   $comment;
+        @comment = grep { /\#/ } @comment;
+
+        foreach $comment (@comment) {
+            $comment =~ s/^\s*?(\#.*)$/$1/gm;
+            chomp $comment;
+        }
+
+        $retval     = \@comment;
+        $debug_desc = 'comment';
+    } else {
+        $retval     = '';
+        $debug_desc = 'unclassified';
+    }
+
+    $self->{parse}{debug_desc} = $debug_desc;
+    $self->{parse}{makecode}   = $retval;
+}
+
+sub _parse_catch_trapped_loop {
+    my $self = shift;
+
+    no warnings 'uninitialized';
+
+    $self->{parse}{trapped_loop}{$self->{parse}{makecode}}++
+      if $self->{parse}{makecode} eq $self->{makecode_prev};
+
+    if ($self->{parse}{trapped_loop}{$self->{parse}{makecode}} > 1) {
+        $self->{Config}{Exec_Makefile} = 1;
+        $self->{Config}{reinit}        = 1;
+        $self->convert;
+        exit;
+    }
+
+    $self->{makecode_prev} = $self->{parse}{makecode};
+}
+
+sub _parse_substitute_makeargs {
+    my $self = shift;
+
+    $self->{parse}{makecode} ||= '';
+
+    foreach my $make (keys %{$self->{Data}{table}}) {
+        if ($self->{parse}{makecode} =~ /\b$make\b/s) {
+            $self->{parse}{makecode} =~ s/$make/$self->{Data}{table}{$make}/;
+       }
+    }
+}
+
+sub _parse_append_makecode {
+    my $self = shift;
+
+    unless (@{$self->{parse}{histargs}||[]}) {
+        push @{$self->{make_code}{args}{begin}}, $self->{parse}{makecode};
+    } else {
+        pop @{$self->{parse}{histargs}}
+          until $self->{Data}{table}{$self->{parse}{histargs}->[-1]};
+
+        push @{$self->{make_code}{args}{$self->{Data}{table}{$self->{parse}{histargs}->[-1]}}},
+               $self->{parse}{makecode};
+    }
+}
+
+sub _parse_register_comment {
+    my $self = shift;
+
+    my $arg     = $self->{parse}{arg};
+    my $comment = $self->{parse}{comment};
+
+    if (defined($comment) && defined($self->{Data}{table}{$arg})) {
+        $self->{make_comments}{$self->{Data}{table}{$arg}} = $comment;
+    }
+}
+
+sub _debug_string_text {
+    my $self = shift;
+
+    my $output = <<EOT;
 Found string ''
 +++++++++++++++
-\$arg: $arg
-\$value: $value
-\$comment: $comment
+\$arg: $self->{parse}{arg}
+\$value: $self->{parse}{value}
+\$comment: $self->{parse}{comment}
 \$remaining args:
-$makefile
+$self->{parse}{makefile}
 
-DEBUG
-        } elsif ($makefile =~ s/$found_array//s) {
-            my ($arg, $values, $comment) = ($1,$2,$3);
-            $comment ||= '';
-            $makeargs{$arg} = [ map { tr/['",]//d; $_ } split /,\s*/, $values ];
-            push @histargs, $arg;
+EOT
+    return $output;
+}
 
-            if (defined($comment) && defined($self->{Data}{table}{$arg})) {
-                $self->{make_comments}{$self->{Data}{table}{$arg}} = $comment;
-            }
+sub _debug_array_text {
+    my $self = shift;
 
-            $self->_debug(<<DEBUG);
+    my @values = @{$self->{parse}{values}};
+
+    my $output = <<EOT;
 Found array []
 ++++++++++++++
-\$arg: $arg
-\$values: $values
-\$comment: $comment
+\$arg: $self->{parse}{arg}
+\$values: @values
+\$comment: $self->{parse}{comment}
 \$remaining args:
-$makefile
+$self->{parse}{makefile}
 
-DEBUG
-        } elsif ($makefile =~ s/$found_hash//s) {
-            my ($arg, $values, $comment) = ($1,$2,$3);
-            $comment ||= '';
-            my @values_debug = split /,\s*/, $values;
-            my @values;
+EOT
+    return $output;
+}
 
-            foreach my $value (@values_debug) {
-                push @values, map { tr/['",]//d; $_ } split /\s*=>\s*/, $value;
-            }
+sub _debug_hash_text {
+    my $self = shift;
 
-            @values_debug = map { "$_\n        " } @values_debug;
-            $makeargs{$arg} = { @values };
-            push @histargs, $arg;
-
-            if (defined($comment) && defined($self->{Data}{table}{$arg})) {
-                $self->{make_comments}{$self->{Data}{table}{$arg}} = $comment;
-            }
-            $self->_debug(<<DEBUG);
+    my $output = <<EOT;
 Found hash {}
 +++++++++++++
-\$key: $arg
-\$values: @values_debug
-\$comment: $comment
+\$key: $self->{parse}{arg}
+\$values: @{$self->{parse}{values}}
+\$comment: $self->{parse}{comment}
 \$remaining args:
-$makefile
+$self->{parse}{makefile}
+EOT
+    return $output;
+}
 
-DEBUG
-        } else {
-            my ($debug_desc, $makecode);
+sub _debug_code_text {
+    my $self = shift;
 
-            if ($makefile =~ s/^\s*(\#.*\s*(?:=>\s*['"]?\w*['"]?))?,?\n//s) {
-                $debug_desc = 'comment';
-                $makecode = $1;
-            } elsif ($makefile =~ s/^\s*(['"]?\w+['"]?\s*=>\s*<<['"]?(.*?)['"]?,.*\2)//s) {
-                $debug_desc = 'here-doc';
-                $makecode = $1;
-            } elsif ($makefile =~ s/^\s*(\(?.*\?.*\:(.*?\),|.*['"],))\s*//s) {
-                $debug_desc = 'ternary operator';
-                $makecode = $1;
-            } elsif ($makefile =~ s/^\s*([\Q$@%\E]\w+)\s*//) {
-                $debug_desc = 'variable';
-                $makecode = $1;
-            } else {
-                $makefile =~ s/^\s*(.*?)\s*//;
-                $debug_desc = 'unknown';
-                $makecode = $1;
-            }
+    my @args;
 
-            no warnings 'uninitialized';
-            $trapped_loop{$makecode}++ if $makecode eq $self->{makefile_prev};
+    if (ref $self->{parse}{makecode} eq 'ARRAY') {
+        push @args, @{$self->{parse}{makecode}};
+    } else {
+        push @args, $self->{parse}{makecode};
+    }
 
-            if ($trapped_loop{$makecode} >= 1) {
-                $self->{Config}{Exec_Makefile} = 1;
-                $self->{Config}{reinit} = 1;
-                $self->convert;
-                exit;
-            }
+    @args = map { "\n$_" } @args if @args > 1;
 
-            $self->{makefile_prev} = $makecode;
-            $self->_debug(<<DEBUG);
+    my $output = <<EOT;
 Found code &
 ++++++++++++
-$debug_desc: $makecode
+$self->{parse}{debug_desc}: @args
 remaining args:
-$makefile
+$self->{parse}{makefile}
 
-DEBUG
-            SUBST: foreach my $make (keys %{$self->{Data}{table}}) {
-                if ($makecode =~ /\b$make\b/s) {
-                    $makecode =~ s/$make/$self->{Data}{table}{$make}/;
-                    last SUBST;
-                }
-            }
-
-            if (@histargs) {
-                pop @histargs until $self->{Data}{table}{$histargs[-1]};
-                push @{$self->{make_code}{$self->{Data}{table}{$histargs[-1]}}}, $makecode;
-            }
-        }
-    }
-    $self->_debug(LEADCHAR."Leaving parse\n\n", 'no_wait');
-    %{$self->{make_args}} = %makeargs;
+EOT
+    return $output;
 }
 
 sub _read_makefile {
     my $self = shift;
 
     my $makefile = File::Slurp::read_file($self->{Config}{Makefile_PL});
-    $makefile =~ s/^(.*)\&?WriteMakefile\s*?\(\s*(.*?)\s*\)\s*?;(.*)$/$2/s;
+    $makefile =~ s/^(.*?)\&?WriteMakefile\s*?\(\s*(.*?)\s*\)\s*?;(.*)$/$2/s;
 
     my $makecode_begin = $1;
     my $makecode_end   = $3;
@@ -498,9 +703,10 @@ sub _read_makefile {
 
 sub _convert {
     my $self = shift;
+
     $self->_insert_args;
 
-    foreach my $arg (keys %{$self->{make_args}}) {
+    foreach my $arg (keys %{$self->{make_args}{args}}) {
         if ($self->{disabled}{$arg}) {
             $self->_do_verbose(LEADCHAR."$arg disabled, skipping\n");
             next;
@@ -509,12 +715,12 @@ sub _convert {
             $self->_do_verbose(LEADCHAR."$arg unknown, skipping\n");
             next;
         }
-        if (ref $self->{make_args}{$arg} eq 'HASH') {
+        if (ref $self->{make_args}{args}{$arg} eq 'HASH') {
             if (ref $self->{Data}{table}->{$arg} eq 'HASH') {
                 # embedded structure
                 my @iterators = ();
                 my $current = $self->{Data}{table}->{$arg};
-                my $value = $self->{make_args}{$arg};
+                my $value = $self->{make_args}{args}{$arg};
                 push @iterators, _iterator($current, $value, keys %$current);
                 while (@iterators) {
                     my $iterator = shift @iterators;
@@ -539,17 +745,18 @@ sub _convert {
                 # flat structure
                 my %tmphash;
                 %{$tmphash{$self->{Data}{table}->{$arg}}} =
-                  map { $_ => $self->{make_args}{$arg}{$_} } keys %{$self->{make_args}{$arg}};
+                  map { $_ => $self->{make_args}{args}{$arg}{$_} } keys %{$self->{make_args}{args}{$arg}};
                 push @{$self->{build_args}}, \%tmphash;
             }
-        } elsif (ref $self->{make_args}{$arg} eq 'ARRAY') { 
-            push @{$self->{build_args}}, { $self->{Data}{table}->{$arg} => $self->{make_args}{$arg} };
-        } elsif (ref $self->{make_args}{$arg} eq '') {
-            push @{$self->{build_args}}, { $self->{Data}{table}->{$arg} => $self->{make_args}{$arg} };
+        } elsif (ref $self->{make_args}{args}{$arg} eq 'ARRAY') { 
+            push @{$self->{build_args}}, { $self->{Data}{table}->{$arg} => $self->{make_args}{args}{$arg} };
+        } elsif (ref $self->{make_args}{args}{$arg} eq '') {
+            push @{$self->{build_args}}, { $self->{Data}{table}->{$arg} => $self->{make_args}{args}{$arg} };
         } else { # unknown type
             warn "$arg - unknown type of argument\n";
         }
     }
+
     $self->_sort_args if @{$self->{Data}{sort_order}};
 }
 
@@ -562,7 +769,7 @@ sub _insert_args {
     while (my ($arg, $value) = each %{$self->{Data}{default_args}}) {
         no warnings 'uninitialized';
 
-        if (exists $self->{make_args}{$build{$arg}}) {
+        if (exists $self->{make_args}{args}{$build{$arg}}) {
             $self->_do_verbose(LEADCHAR."Overriding default \'$arg => $value\'\n");
             next;
         }
@@ -573,6 +780,7 @@ sub _insert_args {
 
         push @insert_args, { $arg => $value };
     }
+
     @{$self->{build_args}} = @insert_args;
 }
 
@@ -594,13 +802,18 @@ sub _sort_args {
     if ($self->{Config}{Use_Native_Order}) {
         no warnings 'uninitialized';
 
+        # Mapping an incremental value to the arguments (keys) in the
+        # order they appear.
         for (my ($i,$s) = 0; $s < @{$self->{make_args_arr}}; $s++) {
+            # Skipping values
             next unless $s % 2 == 0;
+            # Populating table with according M::B arguments and counter
             $native_sortorder{$self->{Data}{table}{$self->{make_args_arr}[$s]}} = $i
               if exists $self->{Data}{table}{$self->{make_args_arr}[$s]};
             $i++;
         }
     }
+
     my %sortorder;
     {
         my %have_args = map { keys %$_ => 1 } @{$self->{build_args}};
@@ -611,15 +824,18 @@ sub _sort_args {
             my %slot;
 
             foreach my $arg (grep $have_args{$_}, @{$self->{Data}{sort_order}}) {
+                # Building sorting table for existing MakeMaker arguments
                 if ($native_sortorder{$arg}) {
                     $sortorder{$arg} = $native_sortorder{$arg};
                     $slot{$native_sortorder{$arg}} = 1;
+                # Inject default arguments at free indexes
                 } else {
                     $i++ while $slot{$i};
                     $sortorder{$arg} = $i++;
                 }
             }
 
+            # Sorting sort table ascending
             my @args = sort { $sortorder{$a} <=> $sortorder{$b} } keys %sortorder;
             $i = 0; %sortorder = map { $_ => $i++ } @args;
 
@@ -629,6 +845,7 @@ sub _sort_args {
             } grep $have_args{$_}, @{$self->{Data}{sort_order}};
         }
     }
+
     my ($is_sorted, @unsorted);
     do {
 
@@ -650,6 +867,7 @@ sub _sort_args {
                   push @{$self->{build_args}},
                     splice(@{$self->{build_args}}, $sortorder{$arg}, 1,
                       splice(@{$self->{build_args}}, $i, 1));
+
                   last SORT;
               }
           }
@@ -667,7 +885,6 @@ sub _dump {
     $Data::Dumper::Terse     = 1;
 
     my $d = Data::Dumper->new(\@{$self->{build_args}});
-
     $self->{buildargs_dumped} = [ $d->Dump ];
 }
 
@@ -677,7 +894,7 @@ sub _write {
     $self->{INDENT} = ' ' x $self->{Config}{Len_Indent};
 
     no warnings 'once';
-    my $fh = IO::File->new(">$self->{Config}{Build_PL}") 
+    my $fh = IO::File->new($self->{Config}{Build_PL}, '>') 
       or die "Can't open $self->{Config}{Build_PL}: $!\n";
 
     my $selold = select($fh);
@@ -697,40 +914,70 @@ sub _write {
 sub _compose_header {
     my $self = shift;
 
-    my ($comments_header, $code_header) = ('','');
+    my ($comments_header, $code_header);
+
     my $note = '# Note: this file has been initially generated by ' . __PACKAGE__ . " $VERSION";
     my $pragmas = "use strict;\nuse warnings;\n";
 
-    if (defined($self->{make_code}{begin})) {
+    # Warnings are thrown for chomp() & regular expressions when enabled
+    no warnings 'uninitialized';
+
+    if (defined $self->{make_code}{begin} || defined $self->{make_code}{end}) {
+        # Removing ExtUtils::MakeMaker dependency
         $self->_do_verbose(LEADCHAR."Removing ExtUtils::MakeMaker as dependency\n");
         $self->{make_code}{begin} =~ s/[ \t]*(?:use|require)\s+ExtUtils::MakeMaker\s*;//;
 
+        # Mapping (prompt|Verbose) calls to Module::Build ones
         if ($self->{make_code}{begin} =~ /(?:prompt|Verbose)\s*\(/s) {
             my $regexp = qr/^(.*?=\s*)(prompt|Verbose)\s*?\(['"](.*)['"]\);$/;
 
-            for my $var (qw(begin end)) {
+            foreach my $var (qw(begin end)) {
                 while ($self->{make_code}{$var} =~ /$regexp/m) {
                     my $replace = $1 . 'Module::Build->' . $2 . '("' . $3 . '");';
                     $self->{make_code}{$var} =~ s/$regexp/$replace/m;
                 }
             }
         }
+
+        # Removing Module::Build::Compat Note
         if ($self->{make_code}{begin} =~ /Module::Build::Compat/) {
             $self->_do_verbose(LEADCHAR."Removing Module::Build::Compat Note\n");
             $self->{make_code}{begin} =~ s/^\#.*Module::Build::Compat.*?\n//s;
         }
-        # Removing pragmas quietly here to ensure that they'll be inserted after
-        # an eventually appearing version requirement.
+
+        # Removing customized MakeMaker subs
+        my $has_MM_sub    = qr/sub MY::/;
+        my $MM_sub_prefix = 'MY::';
+
+        foreach my $var (qw(begin end)) {
+            if ($self->{make_code}{$var} =~ $has_MM_sub) {
+                foreach my $sub (_extract_sub($self->{make_code}{$var}, $MM_sub_prefix)) {
+                    my $quoted_sub = quotemeta $sub;
+                    my ($subname)  = $sub =~ /sub.*?\s+(.*?)\s*\{/;
+
+                    $self->{make_code}{$var} =~ s/$quoted_sub\n//;
+                    $self->_do_verbose(LEADCHAR."Removing sub: '$subname'\n");
+                }
+            }
+        }
+
+        # Removing strict & warnings pragmas quietly here to ensure that they'll
+        # be inserted after an eventually appearing version requirement.
         $self->{make_code}{begin} =~ s/[ \t]*use\s+(?:strict|warnings)\s*;//g;
+
+        # Saving the shebang (interpreter) line
         while ($self->{make_code}{begin} =~ s/^(\#\!?.*?\n)//) {
             $comments_header .= $1;
         }
-        chomp($comments_header);
+        chomp $comments_header;
 
-        while ($self->{make_code}{begin} =~ /(?:use|require)\s+.*?;/) {
-            $self->{make_code}{begin} =~ s/^\n?(.*?;)//s;
+        # Grabbing use & require statements
+        while ($self->{make_code}{begin} =~ /^(?:use|require)\s+(?:[a-z]|[\d\.\_])+?\s*;/m) {
+            $self->{make_code}{begin} =~ s/^\n*(.*?;)//s;
             $code_header .= "$1\n";
         }
+
+        # Adding strict & warnings pragmas
         $self->_do_verbose(LEADCHAR."Adding use strict & use warnings pragmas\n");
 
         if ($code_header =~ /(?:use|require)\s+\d\.[\d_]*\s*;/) { 
@@ -738,12 +985,14 @@ sub _compose_header {
         } else {
             $code_header = $pragmas . $code_header;
         }
-        chomp($code_header);
+        chomp $code_header;
 
+        # Removing leading & trailing newlines
         1 while $self->{make_code}{begin} =~ s/^\n//;
-        chomp($self->{make_code}{begin}) while $self->{make_code}{begin} =~ /\n$/s;
+        chomp $self->{make_code}{begin} while $self->{make_code}{begin} =~ /\n$/s;
     }
 
+    # Constructing the Build.PL header
     $self->{Data}{begin} = $comments_header || $code_header
       ? ($comments_header  =~ /\w/ ? "$comments_header\n" : '') . "$note\n" .
         ($code_header =~ /\w/ ? "\n$code_header\n\n" : "\n") .
@@ -751,6 +1000,36 @@ sub _compose_header {
       : "$note\n\n" . $self->{Data}{begin};
 }
 
+# Albeit Text::Balanced exists, extract_tagged() and friends
+# were (or I?) unable to extract subs.
+sub _extract_sub {
+    my ($text, $pattern) = @_;
+
+    my ($quoted_pattern, %seen, @sub, @subs);
+
+    $quoted_pattern = quotemeta $pattern;
+
+    foreach my $line (split /\n/, $text) {
+        if ($line =~ /^sub $quoted_pattern\w+/s ||
+            $line =~ /^\{/)                        { $seen{begin} = 1 }
+        if ($seen{begin} && $line =~ /^\s*}/)      { $seen{end}   = 1 }
+
+        if ($seen{begin} || $seen{end}) {
+            push @sub, $line;
+        } else {
+            next;
+        }
+
+        if ($seen{end}) {
+            push @subs, join "\n", @sub;
+            @sub = ();
+            @seen{qw(begin end)} = (0,0);
+        }
+    }
+
+    return @subs;
+}
+
 sub _write_begin {
     my $self = shift;
 
@@ -771,13 +1050,26 @@ sub _write_args {
     my $arg;
     my $regex = '$chunk =~ /=> \{/';
 
+    if (@{$self->{make_code}{args}{begin}||[]}) {
+        foreach my $codechunk (@{$self->{make_code}{args}{begin}}) {
+            if (ref $codechunk eq 'ARRAY') {
+                foreach my $code (@$codechunk) {
+                    $self->_do_verbose("$self->{INDENT}$code\n", 2);
+                    print "$self->{INDENT}$code\n";
+                }
+            } else {
+                $self->_do_verbose("$self->{INDENT}$codechunk\n", 2);
+                print "$self->{INDENT}$codechunk\n";
+            }
+        }
+    }
+
     foreach my $chunk (@{$self->{buildargs_dumped}}) {
         # Hash/Array output
         if ($chunk =~ /=> [\{\[]/) {
 
             # Remove redundant parentheses
-            $chunk =~ s/^\{.*?\n(.*(?{eval $regex ? '\}' : '\]'}))\s+\}\s+$/$1/os;
-            Carp::croak $@ if $@;
+            $chunk =~ s/^\{.*?\n(.*(?{ $regex ? '\}' : '\]' }))\s+\}\s+$/$1/os;
 
             # One element per each line
             my @lines;
@@ -791,53 +1083,97 @@ sub _write_args {
 
             foreach (my $i = 0; $i < @lines; $i++) {
                 my $line = $lines[$i];
-                chomp($line);
+                chomp $line;
                 # Remove additional whitespace
                 $line =~ s/^\s{$shorten}(.*)$/$1/o;
 
                 # Quote sub hash keys
                 $line =~ s/^(\s+)([\w:]+)/$1'$2'/ if $line =~ /^\s+/;
 
-                # Add comma where appropriate (version numbers, parentheses)
-                $line .= ',' if $line =~ /[\d+\}\]]$/;
+                # Add comma where appropriate (version numbers, parentheses, brackets)
+                $line .= ',' if $line =~ /[\d+ \} \]] $/x;
 
-                $line =~ s/'(\d|\$\w+)'/$1/g;
+                # (De)quotify numbers, variables & code bits
+                $line =~ s/' \\? ( \d | [\\ \/ \( \) \$ \@ \%]+ \w+) '/$1/gx;
+                $self->_quotify(\$line) if $line =~ /\(/;
 
-                my $output = "$self->{INDENT}$line";
-                $output .= ($i == $#lines && defined($self->{make_comments}{$arg}))
-                  ? "$self->{make_comments}{$arg}\n" : "\n";
+                # Add comma to dequotified key/value pairs
+                my $comma   = ',' if $line =~ /['"](?!,)$/ && $#lines - $i != 1;
+                   $comma ||= '';
 
+                # Construct line output
+                my $output = "$self->{INDENT}$line$comma";
+
+                # Add adhering comments at end of array/hash
+                $output .= ($i == $#lines && defined $self->{make_comments}{$arg})
+                  ? "$self->{make_comments}{$arg}\n"
+                  : "\n";
+
+                # Output line
                 $self->_do_verbose($output, 2);
                 print $output;
             }
-        } else { # Scalar output
-            chomp($chunk);
+        # String output
+        } else {
+            chomp $chunk;
             # Remove redundant parentheses
-            $chunk =~ s/^\{\s+(.*?)\s+\}$/$1/s;
+            $chunk =~ s/^\{\s+(.*?)\s+\}$/$1/sx;
 
-            $chunk =~ s/'(\d|\$\w+)'/$1/g;
+            # (De)quotify numbers, variables & code bits
+            $chunk =~ s/' \\? ( \d | [\\ \/ \( \) \$ \@ \%]+ \w+ ) '/$1/gx;
+            $self->_quotify(\$chunk) if $chunk =~ /\(/;
+
+            # Extract argument (key)
             ($arg) = $chunk =~ /^\s*(\w+)/;
 
+            # Construct line output & add adhering comment
             my $output = "$self->{INDENT}$chunk,";
-            $output .= $self->{make_comments}{$arg} if defined($self->{make_comments}{$arg});
+            $output .= $self->{make_comments}{$arg} if defined $self->{make_comments}{$arg};
 
+            # Output key/value pair
             $self->_do_verbose("$output\n", 2);
             print "$output\n";
         }
 
         no warnings 'uninitialized';
+        my @args;
 
-        if ($self->{make_code}{$arg}) {
-            foreach my $line (@{$self->{make_code}{$arg}}) {
-                $line .= ',' unless $line =~ /^\#/;
+        if ($self->{make_code}{args}{$arg}) {
+            @args = ();
+            foreach my $arg (@{$self->{make_code}{args}{$arg}}) {
+                if (ref $arg eq 'ARRAY') {
+                    push @args, @$arg;
+                } else {
+                    push @args, $arg;
+                }
+            }
 
-                $self->_do_verbose("$self->{INDENT}$line\n", 2);
-                print "$self->{INDENT}$line\n";
+            foreach $arg (@args) {
+                next unless $arg;
+
+                $arg .= ',' unless $arg =~ /^\#/;
+
+                $self->_do_verbose("$self->{INDENT}$arg\n", 2);
+                print "$self->{INDENT}$arg\n";
             }
         }
     }
 }
 
+sub _quotify {
+    my ($self, $string) = @_;
+
+    # Removing single-quotes and escaping backslashes
+    $$string =~ s/(=>\s+?)'/$1/;
+    $$string =~ s/',?$//;
+    $$string =~ s/\\'/'/g; 
+
+    # Double-quoting $(NAME) variables
+    if ($$string =~ /\$\(/) {
+        $$string =~ s/(=>\s+?)(.*)/$1"$2"/;
+    }
+}
+
 sub _write_end {
     my $self = shift;
 
@@ -863,19 +1199,19 @@ sub _subst_makecode {
 sub _add_to_manifest {
     my $self = shift;
 
-    my $fh = IO::File->new("<$self->{Config}{MANIFEST}") 
+    my $fh = IO::File->new($self->{Config}{MANIFEST}, '<')
       or die "Can't open $self->{Config}{MANIFEST}: $!\n";
     my @manifest = <$fh>;
     $fh->close;
 
     my $build_pl = File::Basename::basename($self->{Config}{Build_PL});
 
-    unless (grep { $_ =~ /^$build_pl\s+$/o } @manifest) {
+    unless (grep { /^$build_pl\s+$/o } @manifest) {
         unshift @manifest, "$build_pl\n";
 
-        $fh = IO::File->new(">$self->{Config}{MANIFEST}")
+        $fh = IO::File->new($self->{Config}{MANIFEST}, '>')
           or die "Can't open $self->{Config}{MANIFEST}: $!\n";
-        print $fh sort @manifest;
+        print {$fh} sort @manifest;
         $fh->close;
 
         $self->_do_verbose(LEADCHAR."Added to $self->{Config}{MANIFEST}: $self->{Config}{Build_PL}\n");
@@ -893,12 +1229,19 @@ sub _show_summary {
         [ 'Method: execute', 'method_execute' ],
     );
 
+    local $" = "\n";
+
     foreach my $item (@summary) {
-        next unless defined(@{$self->{summary}{$item->[1]}});
+        next unless @{$self->{summary}{$item->[1]}||[]};
+
         $self->_do_verbose("$item->[0]\n");
         $self->_do_verbose('-' x length($item->[0]), "\n");
         $self->_do_verbose("@{$self->{summary}{$item->[1]}}\n\n");
     }
+
+    my $howmany = @{$self->{summary}->{succeeded}};
+
+    print "Processed $howmany directories\n";
 }
 
 sub _do_verbose {
@@ -914,7 +1257,7 @@ sub _do_verbose {
 
 sub _debug {
     my $self = shift;
-    
+
     if ($self->{Config}{Debug}) {
         pop and my $no_wait = 1 if $_[-1] eq 'no_wait';
         warn @_;
@@ -995,7 +1338,7 @@ $INDENT(
 # end code 
 -
 $INDENT);
-  
+
 $build->create_build_script;
 
 $MAKECODE
@@ -1022,7 +1365,7 @@ Module::Build::Convert - Makefile.PL to Build.PL converter
 
 C<ExtUtils::MakeMaker> has been a de-facto standard for the common distribution of Perl
 modules; C<Module::Build> is expected to supersede C<ExtUtils::MakeMaker> in some time
-(part of the Perl core as of 5.10?)
+(part of the Perl core as of 5.9.4).
 
 The transition takes place slowly, as the converting process manually achieved
 is yet an uncommon practice. The Module::Build::Convert F<Makefile.PL> parser is
@@ -1086,6 +1429,11 @@ switches C<-v> (mode 1) and C<-vv> (mode 2). Default: 0
 Rudimentary debug facility for examining the parsing process.
 Default: 0
 
+=item Process_Code
+
+Process code embedded within the arguments list.
+Default: 0
+
 =item Use_Native_Order
 
 Native sorting order. If set to 1, the native sorting order of
